@@ -19,12 +19,18 @@
 
 package org.apache.samza.operators;
 
+import java.time.Duration;
+import java.util.Collection;
+import java.util.function.Function;
+
 import org.apache.samza.SamzaException;
 import org.apache.samza.operators.functions.FilterFunction;
 import org.apache.samza.operators.functions.FlatMapFunction;
 import org.apache.samza.operators.functions.JoinFunction;
 import org.apache.samza.operators.functions.MapFunction;
 import org.apache.samza.operators.functions.SinkFunction;
+import org.apache.samza.operators.functions.StreamTableJoinFunction;
+import org.apache.samza.operators.spec.BroadcastOperatorSpec;
 import org.apache.samza.operators.spec.JoinOperatorSpec;
 import org.apache.samza.operators.spec.OperatorSpec;
 import org.apache.samza.operators.spec.OperatorSpec.OpCode;
@@ -32,18 +38,18 @@ import org.apache.samza.operators.spec.OperatorSpecs;
 import org.apache.samza.operators.spec.OutputOperatorSpec;
 import org.apache.samza.operators.spec.OutputStreamImpl;
 import org.apache.samza.operators.spec.PartitionByOperatorSpec;
+import org.apache.samza.operators.spec.SendToTableOperatorSpec;
 import org.apache.samza.operators.spec.SinkOperatorSpec;
 import org.apache.samza.operators.spec.StreamOperatorSpec;
+import org.apache.samza.operators.spec.StreamTableJoinOperatorSpec;
 import org.apache.samza.operators.stream.IntermediateMessageStreamImpl;
 import org.apache.samza.operators.windows.Window;
 import org.apache.samza.operators.windows.WindowPane;
 import org.apache.samza.operators.windows.internal.WindowInternal;
 import org.apache.samza.serializers.KVSerde;
 import org.apache.samza.serializers.Serde;
-
-import java.time.Duration;
-import java.util.Collection;
-import java.util.function.Function;
+import org.apache.samza.table.Table;
+import org.apache.samza.table.TableSpec;
 
 
 /**
@@ -137,6 +143,16 @@ public class MessageStreamImpl<M> implements MessageStream<M> {
     return new MessageStreamImpl<>(this.graph, joinOpSpec);
   }
 
+  @Override
+  public <K, R extends KV, JM> MessageStream<JM> join(Table<R> table,
+      StreamTableJoinFunction<? extends K, ? super M, ? super R, ? extends JM> joinFn) {
+    TableSpec tableSpec = ((TableImpl) table).getTableSpec();
+    StreamTableJoinOperatorSpec<K, M, R, JM> joinOpSpec = OperatorSpecs.createStreamTableJoinOperatorSpec(
+        tableSpec, (StreamTableJoinFunction<K, M, R, JM>) joinFn, this.graph.getNextOpId(OpCode.JOIN));
+    this.operatorSpec.registerNextOperatorSpec(joinOpSpec);
+    return new MessageStreamImpl<>(this.graph, joinOpSpec);
+  }
+
   @Override
   public MessageStream<M> merge(Collection<? extends MessageStream<? extends M>> otherStreams) {
     if (otherStreams.isEmpty()) return this;
@@ -176,4 +192,27 @@ public class MessageStreamImpl<M> implements MessageStream<M> {
   protected OperatorSpec<?, M> getOperatorSpec() {
     return this.operatorSpec;
   }
+
+  @Override
+  public <K, V> void sendTo(Table<KV<K, V>> table) {
+    SendToTableOperatorSpec<K, V> op = OperatorSpecs.createSendToTableOperatorSpec(
+        this.operatorSpec, ((TableImpl) table).getTableSpec(), this.graph.getNextOpId(OpCode.SEND_TO));
+    this.operatorSpec.registerNextOperatorSpec(op);
+  }
+
+  @Override
+  public MessageStream<M> broadcast(Serde<M> serde, String userDefinedId) {
+    String opId = this.graph.getNextOpId(OpCode.BROADCAST, userDefinedId);
+    IntermediateMessageStreamImpl<M> intermediateStream = this.graph.getIntermediateStream(opId, serde, true);
+    BroadcastOperatorSpec<M> broadcastOperatorSpec =
+        OperatorSpecs.createBroadCastOperatorSpec(intermediateStream.getOutputStream(), opId);
+    this.operatorSpec.registerNextOperatorSpec(broadcastOperatorSpec);
+    return intermediateStream;
+  }
+
+  @Override
+  public MessageStream<M> broadcast(String userDefinedId) {
+    return broadcast(null, userDefinedId);
+  }
+
 }
