@@ -19,6 +19,10 @@
 
 package org.apache.samza.operators.spec;
 
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.function.Function;
+
 import org.apache.samza.config.Config;
 import org.apache.samza.operators.KV;
 import org.apache.samza.operators.functions.FilterFunction;
@@ -26,15 +30,13 @@ import org.apache.samza.operators.functions.FlatMapFunction;
 import org.apache.samza.operators.functions.JoinFunction;
 import org.apache.samza.operators.functions.MapFunction;
 import org.apache.samza.operators.functions.SinkFunction;
+import org.apache.samza.operators.functions.StreamTableJoinFunction;
 import org.apache.samza.operators.windows.internal.WindowInternal;
 import org.apache.samza.serializers.Serde;
 import org.apache.samza.system.StreamSpec;
+import org.apache.samza.table.TableSpec;
 import org.apache.samza.task.TaskContext;
 
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.function.Function;
-
 
 /**
  * Factory methods for creating {@link OperatorSpec} instances.
@@ -93,7 +95,7 @@ public class OperatorSpecs {
       public void close() {
         mapFn.close();
       }
-    }, OperatorSpec.OpCode.MAP, opId);
+    }, mapFn, OperatorSpec.OpCode.MAP, opId);
   }
 
   /**
@@ -127,7 +129,7 @@ public class OperatorSpecs {
       public void close() {
         filterFn.close();
       }
-    }, OperatorSpec.OpCode.FILTER, opId);
+    }, filterFn, OperatorSpec.OpCode.FILTER, opId);
   }
 
   /**
@@ -141,7 +143,7 @@ public class OperatorSpecs {
    */
   public static <M, OM> StreamOperatorSpec<M, OM> createFlatMapOperatorSpec(
       FlatMapFunction<? super M, ? extends OM> flatMapFn, String opId) {
-    return new StreamOperatorSpec<>((FlatMapFunction<M, OM>) flatMapFn, OperatorSpec.OpCode.FLAT_MAP, opId);
+    return new StreamOperatorSpec<>((FlatMapFunction<M, OM>) flatMapFn, flatMapFn, OperatorSpec.OpCode.FLAT_MAP, opId);
   }
 
   /**
@@ -240,6 +242,52 @@ public class OperatorSpecs {
             this.add(message);
           }
         },
-        OperatorSpec.OpCode.MERGE, opId);
+        null, OperatorSpec.OpCode.MERGE, opId);
   }
+
+  /**
+   * Creates a {@link StreamTableJoinOperatorSpec} with a join function.
+   *
+   * @param tableSpec the table spec for the table on the right side of the join
+   * @param joinFn the user-defined join function to get join keys and results
+   * @param opId the unique ID of the operator
+   * @param <K> the type of join key
+   * @param <M> the type of input messages
+   * @param <R> the type of table record
+   * @param <JM> the type of the join result
+   * @return the {@link StreamTableJoinOperatorSpec}
+   */
+  public static <K, M, R, JM> StreamTableJoinOperatorSpec<K, M, R, JM> createStreamTableJoinOperatorSpec(
+      TableSpec tableSpec, StreamTableJoinFunction<K, M, R, JM> joinFn, String opId) {
+    return new StreamTableJoinOperatorSpec(tableSpec, joinFn, opId);
+  }
+
+  /**
+   * Creates a {@link SendToTableOperatorSpec} with a key extractor and a value extractor function,
+   * the type of incoming message is expected to be KV&#60;K, V&#62;.
+   *
+   * @param inputOpSpec the operator spec for the input stream
+   * @param tableSpec the table spec for the underlying table
+   * @param opId the unique ID of the operator
+   * @param <K> the type of the table record key
+   * @param <V> the type of the table record value
+   * @return the {@link SendToTableOperatorSpec}
+   */
+  public static <K, V> SendToTableOperatorSpec<K, V> createSendToTableOperatorSpec(
+      OperatorSpec<?, KV<K, V>> inputOpSpec, TableSpec tableSpec, String opId) {
+    return new SendToTableOperatorSpec(inputOpSpec, tableSpec, opId);
+  }
+
+  /**
+   * Creates a {@link BroadcastOperatorSpec} for the Broadcast operator.
+   * @param outputStream the {@link OutputStreamImpl} to send messages to
+   * @param opId the unique ID of the operator
+   * @param <M> the type of input message
+   * @return the {@link BroadcastOperatorSpec}
+   */
+  public static <M> BroadcastOperatorSpec<M> createBroadCastOperatorSpec(
+      OutputStreamImpl<M> outputStream, String opId) {
+    return new BroadcastOperatorSpec<>(outputStream, opId);
+  }
+
 }
