@@ -94,8 +94,6 @@ public class ZkUtils {
     return currentGeneration.get();
   }
 
-
-
   public ZkUtils(ZkKeyBuilder zkKeyBuilder, ZkClient zkClient, int connectionTimeoutMs, MetricsRegistry metricsRegistry) {
     this.keyBuilder = zkKeyBuilder;
     this.connectionTimeoutMs = connectionTimeoutMs;
@@ -191,15 +189,18 @@ public class ZkUtils {
    * Fetches all the ephemeral processor nodes of a standalone job from zookeeper.
    * @return a list of {@link ProcessorNode}, where each ProcessorNode represents a registered stream processor.
    */
-  private List<ProcessorNode> getAllProcessorNodes() {
+  List<ProcessorNode> getAllProcessorNodes() {
     List<String> processorZNodes = getSortedActiveProcessorsZnodes();
     LOG.debug("Active ProcessorZNodes in zookeeper: {}.", processorZNodes);
-    return processorZNodes.stream()
-                          .map(processorZNode -> {
-                              String ephemeralProcessorPath = String.format("%s/%s", keyBuilder.getProcessorsPath(), processorZNode);
-                              String data = readProcessorData(ephemeralProcessorPath);
-                              return new ProcessorNode(new ProcessorData(data), ephemeralProcessorPath);
-                            }).collect(Collectors.toList());
+    List<ProcessorNode> processorNodes = new ArrayList<>();
+    for (String processorZNode: processorZNodes) {
+      String ephemeralProcessorPath = String.format("%s/%s", keyBuilder.getProcessorsPath(), processorZNode);
+      String data = readProcessorData(ephemeralProcessorPath);
+      if (data != null) {
+        processorNodes.add(new ProcessorNode(new ProcessorData(data), ephemeralProcessorPath));
+      }
+    }
+    return processorNodes;
   }
 
   /**
@@ -223,12 +224,10 @@ public class ZkUtils {
    * @throws SamzaException when fullPath doesn't exist in zookeeper
    * or problems with connecting to zookeeper.
    */
-  String readProcessorData(String fullPath) {
+  private String readProcessorData(String fullPath) {
     try {
-      String data = zkClient.readData(fullPath, false);
-      if (metrics != null) {
-        metrics.reads.inc();
-      }
+      String data = zkClient.readData(fullPath, true);
+      metrics.reads.inc();
       return data;
     } catch (Exception e) {
       throw new SamzaException(String.format("Cannot read ZK node: %s", fullPath), e);
@@ -254,7 +253,10 @@ public class ZkUtils {
     if (znodeIds.size() > 0) {
       for (String child : znodeIds) {
         String fullPath = String.format("%s/%s", processorPath, child);
-        processorIds.add(new ProcessorData(readProcessorData(fullPath)).getProcessorId());
+        String processorData = readProcessorData(fullPath);
+        if (processorData != null) {
+          processorIds.add(new ProcessorData(processorData).getProcessorId());
+        }
       }
       Collections.sort(processorIds);
       LOG.info("Found these children - " + znodeIds);
@@ -298,7 +300,13 @@ public class ZkUtils {
   }
 
   public void close() throws ZkInterruptedException {
-    zkClient.close();
+    try {
+      zkClient.close();
+    } catch (ZkInterruptedException e) {
+      // Swallowing due to occurrence in the last stage of lifecycle (Not actionable) and clear the interrupted status.
+      Thread.interrupted();
+      LOG.warn("Ignoring the exception when closing the zookeeper client.", e);
+    }
   }
 
   /**
@@ -412,6 +420,29 @@ public class ZkUtils {
     return jobModelVersion;
   }
 
+  /**
+   * Generates the next JobModel version that should be used by a processor group in a rebalancing phase
+   * for coordination.
+   * @param currentJobModelVersion the current version of JobModel.
+   * @return the next JobModel version.
+   */
+  public String getNextJobModelVersion(String currentJobModelVersion) {
+    if (currentJobModelVersion == null) {
+      return  "1";
+    } else {
+      /**
+       * There's inconsistency between the maximum published jobModel version and value stored in jobModelVersion
+       * zookeeper node. Short term fix is to read all published jobModel versions and choose the maximum version. If there's a
+       * inconsistency, update the jobModelVersionPath with maximum published jobModelVersion.
+       */
+      List<String> publishedJobModelVersions = zkClient.getChildren(keyBuilder.getJobModelPathPrefix());
+      metrics.reads.inc(publishedJobModelVersions.size());
+      String maxPublishedJMVersion = publishedJobModelVersions.stream()
+                                                              .max(Comparator.comparingInt(Integer::valueOf)).orElse("0");
+      return Integer.toString(Math.max(Integer.valueOf(currentJobModelVersion), Integer.valueOf(maxPublishedJMVersion)) + 1);
+    }
+  }
+
   /**
    * publish the version number of the next JobModel
    * @param oldVersion - used to validate, that no one has changed the version in the meanwhile.
@@ -562,7 +593,7 @@ public class ZkUtils {
   /**
    * Represents zookeeper processor node.
    */
-  private static class ProcessorNode {
+  static class ProcessorNode {
     private final ProcessorData processorData;
 
     // Ex: /test/processors/0000000000
