@@ -21,7 +21,6 @@ package org.apache.samza.config
 
 
 import java.util.concurrent.TimeUnit
-import org.apache.samza.SamzaException
 import scala.collection.JavaConverters._
 import org.apache.samza.util.Logging
 import org.apache.samza.util.Util
@@ -51,7 +50,7 @@ class StorageConfig(config: Config) extends ScalaMapConfig(config) with Logging
   def getStorageMsgSerde(name: String) = getOption(StorageConfig.MSG_SERDE format name)
 
   def getAccessLogEnabled(storeName: String) = {
-    getBoolean(ACCESSLOG_ENABLED format storeName, false)
+    new JavaStorageConfig(config).getAccessLogEnabled(storeName)
   }
 
   def getChangelogStream(name: String) = {
@@ -61,11 +60,11 @@ class StorageConfig(config: Config) extends ScalaMapConfig(config) with Logging
 
   //Returns the accesslog stream name given a changelog stream name
   def getAccessLogStream(changeLogStream: String) = {
-    changeLogStream + "-" + ACCESSLOG_STREAM_SUFFIX
+    new JavaStorageConfig(config).getAccessLogStream(changeLogStream)
   }
 
   def getAccessLogSamplingRatio(storeName: String) = {
-    getInt(ACCESSLOG_SAMPLING_RATIO format storeName, DEFAULT_ACCESSLOG_SAMPLING_RATIO)
+    new JavaStorageConfig(config).getAccessLogSamplingRatio(storeName)
   }
 
   def getChangeLogDeleteRetentionInMs(storeName: String) = {
