@@ -37,6 +37,7 @@ object StreamConfig {
   val BOOTSTRAP =               SAMZA_PROPERTY + "bootstrap"
   val PRIORITY =                SAMZA_PROPERTY + "priority"
   val IS_INTERMEDIATE =         SAMZA_PROPERTY + "intermediate"
+  val DELETE_COMMITTED_MESSAGES = SAMZA_PROPERTY + "delete.committed.messages"
   val IS_BOUNDED =              SAMZA_PROPERTY + "bounded"
 
   // We don't want any external dependencies on these patterns while both exist. Use getProperty to ensure proper values.
@@ -47,9 +48,11 @@ object StreamConfig {
   val SYSTEM_FOR_STREAM_ID = STREAM_ID_PREFIX + SYSTEM
   val PHYSICAL_NAME_FOR_STREAM_ID = STREAM_ID_PREFIX + PHYSICAL_NAME
   val IS_INTERMEDIATE_FOR_STREAM_ID = STREAM_ID_PREFIX + IS_INTERMEDIATE
+  val DELETE_COMMITTED_MESSAGES_FOR_STREAM_ID = STREAM_ID_PREFIX + DELETE_COMMITTED_MESSAGES
   val IS_BOUNDED_FOR_STREAM_ID = STREAM_ID_PREFIX + IS_BOUNDED
   val PRIORITY_FOR_STREAM_ID = STREAM_ID_PREFIX + PRIORITY
   val CONSUMER_OFFSET_DEFAULT_FOR_STREAM_ID = STREAM_ID_PREFIX + CONSUMER_OFFSET_DEFAULT
+  val BOOTSTRAP_FOR_STREAM_ID = STREAM_ID_PREFIX + BOOTSTRAP
 
   implicit def Config2Stream(config: Config) = new StreamConfig(config)
 }
@@ -165,10 +168,19 @@ class StreamConfig(config: Config) extends ScalaMapConfig(config) with Logging {
    * @param streamId  the identifier for the stream in the config.
    * @return          true if the stream is intermediate
    */
-  def getIsIntermediate(streamId: String) = {
+  def getIsIntermediateStream(streamId: String) = {
     getBoolean(StreamConfig.IS_INTERMEDIATE_FOR_STREAM_ID format streamId, false)
   }
 
+  /**
+    * Gets the boolean flag of whether the committed messages specified streamId can be deleted
+    * @param streamId  the identifier for the stream in the config.
+    * @return          true if the committed messages of the stream can be deleted
+    */
+  def getDeleteCommittedMessages(streamId: String) = {
+    getBoolean(StreamConfig.DELETE_COMMITTED_MESSAGES_FOR_STREAM_ID format streamId, false)
+  }
+
   def getIsBounded(streamId: String) = {
     getBoolean(StreamConfig.IS_BOUNDED_FOR_STREAM_ID format streamId, false)
   }
