@@ -19,6 +19,8 @@
 
 package org.apache.samza.config
 
+import org.apache.samza.checkpoint.CheckpointManager
+import org.apache.samza.metrics.MetricsRegistry
 import org.apache.samza.system.SystemStream
 import org.apache.samza.util.{Logging, Util}
 
@@ -32,7 +34,7 @@ object TaskConfig {
   val COMMAND_BUILDER = "task.command.class" // streaming.task-factory-class
   val LIFECYCLE_LISTENERS = "task.lifecycle.listeners" // li-generator,foo
   val LIFECYCLE_LISTENER = "task.lifecycle.listener.%s.class" // task.lifecycle.listener.li-generator.class
-  val CHECKPOINT_MANAGER_FACTORY = "task.checkpoint.factory" // class name to use when sending offset checkpoints
+  val CHECKPOINT_MANAGER_FACTORY = TaskConfigJava.CHECKPOINT_MANAGER_FACTORY // class name to use when sending offset checkpoints
   val MESSAGE_CHOOSER_CLASS_NAME = "task.chooser.class"
   val DROP_DESERIALIZATION_ERROR = "task.drop.deserialization.errors" // define whether drop the messages or not when deserialization fails
   val DROP_SERIALIZATION_ERROR = "task.drop.serialization.errors" // define whether drop the messages or not when serialization fails
@@ -42,6 +44,13 @@ object TaskConfig {
   val MAX_CONCURRENCY = "task.max.concurrency" // max number of concurrent process for a AsyncStreamTask
   val CALLBACK_TIMEOUT_MS = "task.callback.timeout.ms"  // timeout period for triggering a callback
   val ASYNC_COMMIT = "task.async.commit" // to enable async commit in a AsyncStreamTask
+  val MAX_IDLE_MS = "task.max.idle.ms"  // maximum time to wait for a task worker to complete when there are no new messages to handle
+
+  val DEFAULT_WINDOW_MS: Long = -1L
+  val DEFAULT_COMMIT_MS = 60000L
+  val DEFAULT_CALLBACK_TIMEOUT_MS: Long = -1L
+  val DEFAULT_MAX_CONCURRENCY: Int = 1
+  val DEFAULT_MAX_IDLE_MS: Long = 10
 
   /**
    * Samza's container polls for more messages under two conditions. The first
@@ -64,6 +73,8 @@ object TaskConfig {
 }
 
 class TaskConfig(config: Config) extends ScalaMapConfig(config) with Logging {
+  val javaTaskConfig = new TaskConfigJava(config)
+
   def getInputStreams = getOption(TaskConfig.INPUT_STREAMS) match {
     case Some(streams) => if (streams.length > 0) {
       streams.split(",").map(systemStreamNames => {
@@ -75,14 +86,14 @@ class TaskConfig(config: Config) extends ScalaMapConfig(config) with Logging {
     case _ => Set[SystemStream]()
   }
 
-  def getWindowMs: Option[Long] = getOption(TaskConfig.WINDOW_MS) match {
-    case Some(ms) => Some(ms.toLong)
-    case _ => None
+  def getWindowMs: Long = getOption(TaskConfig.WINDOW_MS) match {
+    case Some(ms) => ms.toLong
+    case _ => TaskConfig.DEFAULT_WINDOW_MS
   }
 
-  def getCommitMs: Option[Long] = getOption(TaskConfig.COMMIT_MS) match {
-    case Some(ms) => Some(ms.toLong)
-    case _ => None
+  def getCommitMs: Long = getOption(TaskConfig.COMMIT_MS) match {
+    case Some(ms) => ms.toLong
+    case _ => TaskConfig.DEFAULT_COMMIT_MS
   }
 
   def getShutdownMs: Option[Long] = getOption(TaskConfig.SHUTDOWN_MS) match {
@@ -100,7 +111,11 @@ class TaskConfig(config: Config) extends ScalaMapConfig(config) with Logging {
 
   def getCommandClass(defaultValue: String) = getOrElse(TaskConfig.COMMAND_BUILDER, defaultValue)
 
-  def getCheckpointManagerFactory() = getOption(TaskConfig.CHECKPOINT_MANAGER_FACTORY)
+  def getCheckpointManagerFactory() = Option(javaTaskConfig.getCheckpointManagerFactoryName)
+
+  def getCheckpointManager(metricsRegistry: MetricsRegistry): Option[CheckpointManager] = {
+    Option(javaTaskConfig.getCheckpointManager(metricsRegistry))
+  }
 
   def getMessageChooserClass = getOption(TaskConfig.MESSAGE_CHOOSER_CLASS_NAME)
 
@@ -123,23 +138,28 @@ class TaskConfig(config: Config) extends ScalaMapConfig(config) with Logging {
     }
   }
 
-  def getMaxConcurrency: Option[Int] = getOption(TaskConfig.MAX_CONCURRENCY) match {
-    case Some(count) => Some(count.toInt)
-    case _ => None
+  def getMaxConcurrency: Int = getOption(TaskConfig.MAX_CONCURRENCY) match {
+    case Some(count) => count.toInt
+    case _ => TaskConfig.DEFAULT_MAX_CONCURRENCY
   }
 
-  def getCallbackTimeoutMs: Option[Long] = getOption(TaskConfig.CALLBACK_TIMEOUT_MS) match {
-    case Some(ms) => Some(ms.toLong)
-    case _ => None
+  def getCallbackTimeoutMs: Long = getOption(TaskConfig.CALLBACK_TIMEOUT_MS) match {
+    case Some(ms) => ms.toLong
+    case _ => TaskConfig.DEFAULT_CALLBACK_TIMEOUT_MS
   }
 
-  def getAsyncCommit: Option[Boolean] = getOption(TaskConfig.ASYNC_COMMIT) match {
-    case Some(asyncCommit) => Some(asyncCommit.toBoolean)
-    case _ => None
+  def getAsyncCommit: Boolean = getOption(TaskConfig.ASYNC_COMMIT) match {
+    case Some(asyncCommit) => asyncCommit.toBoolean
+    case _ => false
   }
 
-  def isAutoCommitEnabled() = getOption(TaskConfig.COMMIT_MS) match {
+  def isAutoCommitEnabled: Boolean = getOption(TaskConfig.COMMIT_MS) match {
     case Some(commitMs) => commitMs.toInt > 0
-    case _ => true
+    case _ => TaskConfig.DEFAULT_COMMIT_MS > 0
+  }
+
+  def getMaxIdleMs: Long = getOption(TaskConfig.MAX_IDLE_MS) match {
+    case Some(ms) => ms.toLong
+    case _ => TaskConfig.DEFAULT_MAX_IDLE_MS
   }
 }
