@@ -19,6 +19,7 @@
 
 package org.apache.samza.clustermanager;
 
+import com.google.common.collect.ImmutableMap;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.JobConfig;
 import org.apache.samza.config.MapConfig;
@@ -48,11 +49,11 @@ import static org.mockito.Mockito.when;
 
 public class TestContainerProcessManager {
   private final MockClusterResourceManagerCallback callback = new MockClusterResourceManagerCallback();
-  private final MockClusterResourceManager manager = new MockClusterResourceManager(callback);
+  private final MockClusterResourceManager clusterResourceManager = new MockClusterResourceManager(callback);
 
   private static volatile boolean isRunning = false;
 
-  private Map<String, String> configVals = new HashMap<String, String>()  {
+  private Map<String, String> configVals = new HashMap<String, String>() {
     {
       put("cluster-manager.container.count", "1");
       put("cluster-manager.container.retry.count", "1");
@@ -86,15 +87,16 @@ public class TestContainerProcessManager {
 
   private SamzaApplicationState state = null;
 
-  private JobModelManager getJobModelManagerWithHostAffinity(int containerCount) {
+  private JobModelManager getJobModelManagerWithHostAffinity(Map<String, String> containerIdToHost) {
     Map<String, Map<String, String>> localityMap = new HashMap<>();
-    localityMap.put("0", new HashMap<String, String>() { {
-        put(SetContainerHostMapping.HOST_KEY, "abc");
-      } });
+    containerIdToHost.forEach((containerId, host) -> {
+        localityMap.put(containerId, ImmutableMap.of(SetContainerHostMapping.HOST_KEY, containerIdToHost.get(containerId)));
+      });
     LocalityManager mockLocalityManager = mock(LocalityManager.class);
     when(mockLocalityManager.readContainerLocality()).thenReturn(localityMap);
 
-    return JobModelManagerTestUtil.getJobModelManagerWithLocalityManager(getConfig(), containerCount, mockLocalityManager, this.server);
+    return JobModelManagerTestUtil.getJobModelManagerWithLocalityManager(getConfig(),
+        containerIdToHost.size(), mockLocalityManager, this.server);
   }
 
   private JobModelManager getJobModelManagerWithoutHostAffinity(int containerCount) {
@@ -125,7 +127,7 @@ public class TestContainerProcessManager {
         new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
 
     AbstractContainerAllocator allocator =
@@ -140,12 +142,12 @@ public class TestContainerProcessManager {
     conf.put("cluster-manager.container.memory.mb", "500");
     conf.put("cluster-manager.container.cpu.cores", "5");
 
-    state = new SamzaApplicationState(getJobModelManagerWithHostAffinity(1));
+    state = new SamzaApplicationState(getJobModelManagerWithHostAffinity(ImmutableMap.of("0", "host1")));
     taskManager = new ContainerProcessManager(
         new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
 
     allocator =
@@ -165,11 +167,11 @@ public class TestContainerProcessManager {
         new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
 
     MockContainerAllocator allocator = new MockContainerAllocator(
-        manager,
+        clusterResourceManager,
         conf,
         state);
 
@@ -203,11 +205,11 @@ public class TestContainerProcessManager {
     Config conf = getConfig();
     state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
 
-    ContainerProcessManager taskManager =  new ContainerProcessManager(
+    ContainerProcessManager taskManager = new ContainerProcessManager(
         new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
     taskManager.start();
 
@@ -228,16 +230,16 @@ public class TestContainerProcessManager {
     state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
 
     ContainerProcessManager taskManager = new ContainerProcessManager(
-            new MapConfig(conf),
-            state,
-            new MetricsRegistryMap(),
-            manager
+        new MapConfig(conf),
+        state,
+        new MetricsRegistryMap(),
+        clusterResourceManager
     );
 
     MockContainerAllocator allocator = new MockContainerAllocator(
-            manager,
-            conf,
-            state);
+        clusterResourceManager,
+        conf,
+        state);
 
     getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
 
@@ -250,7 +252,7 @@ public class TestContainerProcessManager {
     assertFalse(taskManager.shouldShutdown());
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
 
-    SamzaResource container = new SamzaResource(1, 1024, "abc", "id0");
+    SamzaResource container = new SamzaResource(1, 1024, "host1", "id0");
     taskManager.onResourceAllocated(container);
 
     // Allow container to run and update state
@@ -258,6 +260,7 @@ public class TestContainerProcessManager {
     if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
       fail("timed out waiting for the containers to start");
     }
+    taskManager.onStreamProcessorLaunchSuccess(container);
     assertFalse(taskManager.shouldShutdown());
 
     taskManager.onResourceCompleted(new SamzaResourceStatus("id0", "diagnostics", SamzaResourceStatus.SUCCESS));
@@ -278,11 +281,11 @@ public class TestContainerProcessManager {
         new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
 
     MockContainerAllocator allocator = new MockContainerAllocator(
-        manager,
+        clusterResourceManager,
         conf,
         state);
 
@@ -298,14 +301,14 @@ public class TestContainerProcessManager {
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
 
 
-    SamzaResource container = new SamzaResource(1, 1024, "abc", "id0");
+    SamzaResource container = new SamzaResource(1, 1024, "host1", "id0");
     taskManager.onResourceAllocated(container);
 
     // Allow container to run and update state
     if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
       fail("timed out waiting for the containers to start");
     }
-
+    taskManager.onStreamProcessorLaunchSuccess(container);
     // Create first container failure
     taskManager.onResourceCompleted(new SamzaResourceStatus(container.getResourceID(), "diagnostics", 1));
 
@@ -316,8 +319,8 @@ public class TestContainerProcessManager {
 
     assertFalse(taskManager.shouldShutdown());
     assertFalse(state.jobHealthy.get());
-    assertEquals(2, manager.resourceRequests.size());
-    assertEquals(0, manager.releasedResources.size());
+    assertEquals(2, clusterResourceManager.resourceRequests.size());
+    assertEquals(0, clusterResourceManager.releasedResources.size());
 
     taskManager.onResourceAllocated(container);
 
@@ -325,7 +328,7 @@ public class TestContainerProcessManager {
     if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
       fail("timed out waiting for the containers to start");
     }
-
+    taskManager.onStreamProcessorLaunchSuccess(container);
     assertTrue(state.jobHealthy.get());
 
     // Create a second failure
@@ -334,8 +337,8 @@ public class TestContainerProcessManager {
 
     // The above failure should trigger a job shutdown because our retry count is set to 1
     assertEquals(0, allocator.getContainerRequestState().numPendingRequests());
-    assertEquals(2, manager.resourceRequests.size());
-    assertEquals(0, manager.releasedResources.size());
+    assertEquals(2, clusterResourceManager.resourceRequests.size());
+    assertEquals(0, clusterResourceManager.releasedResources.size());
     assertFalse(state.jobHealthy.get());
     assertTrue(taskManager.shouldShutdown());
     assertEquals(SamzaApplicationState.SamzaAppStatus.FAILED, state.status);
@@ -352,25 +355,25 @@ public class TestContainerProcessManager {
     state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
 
     ContainerProcessManager taskManager = new ContainerProcessManager(
-            new MapConfig(conf),
-            state,
-            new MetricsRegistryMap(),
-            manager
+        new MapConfig(conf),
+        state,
+        new MetricsRegistryMap(),
+        clusterResourceManager
     );
 
     MockContainerAllocator allocator = new MockContainerAllocator(
-            manager,
-            conf,
-            state);
+        clusterResourceManager,
+        conf,
+        state);
     getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
 
     Thread thread = new Thread(allocator);
     getPrivateFieldFromTaskManager("allocatorThread", taskManager).set(taskManager, thread);
 
-    // Start the task manager
+    // Start the task clusterResourceManager
     taskManager.start();
 
-    SamzaResource container = new SamzaResource(1, 1000, "abc", "id1");
+    SamzaResource container = new SamzaResource(1, 1000, "host1", "id1");
     taskManager.onResourceAllocated(container);
 
     // Allow container to run and update state
@@ -388,6 +391,95 @@ public class TestContainerProcessManager {
     assertEquals(state.redundantNotifications.get(), 1);
   }
 
+  @Test
+  public void testRerequestOnAnyHostIfContainerStartFails() throws Exception {
+    state = new SamzaApplicationState(getJobModelManagerWithHostAffinity(ImmutableMap.of("1", "host1")));
+    Map<String, String> configMap = new HashMap<>();
+    configMap.putAll(getConfig());
+
+    MockContainerAllocator allocator = new MockContainerAllocator(
+        clusterResourceManager,
+        new MapConfig(config),
+        state);
+
+    ContainerProcessManager manager = new ContainerProcessManager(config, state, new MetricsRegistryMap(), allocator,
+        clusterResourceManager);
+
+    manager.start();
+    SamzaResource resource = new SamzaResource(1, 1024, "host1", "resource-1");
+    state.pendingContainers.put("1", resource);
+    Assert.assertEquals(clusterResourceManager.resourceRequests.size(), 1);
+    manager.onStreamProcessorLaunchFailure(resource, new Exception("cannot launch container!"));
+    Assert.assertEquals(clusterResourceManager.resourceRequests.size(), 2);
+    Assert.assertEquals(clusterResourceManager.resourceRequests.get(1).getHost(), ResourceRequestState.ANY_HOST);
+  }
+
+  @Test
+  public void testAllBufferedResourcesAreUtilized() throws Exception {
+    Map<String, String> config = new HashMap<>();
+    config.putAll(getConfigWithHostAffinity());
+    config.put("cluster-manager.container.count", "2");
+    Config cfg = new MapConfig(config);
+    // 1. Request two containers on hosts - host1 and host2
+    state = new SamzaApplicationState(getJobModelManagerWithHostAffinity(ImmutableMap.of("0", "host1",
+        "1", "host2")));
+
+    ContainerProcessManager taskManager = new ContainerProcessManager(
+        cfg,
+        state,
+        new MetricsRegistryMap(),
+        clusterResourceManager
+    );
+
+    MockHostAwareContainerAllocator allocator = new MockHostAwareContainerAllocator(
+        clusterResourceManager,
+        cfg,
+        state);
+    getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
+
+    Thread thread = new Thread(allocator);
+    getPrivateFieldFromTaskManager("allocatorThread", taskManager).set(taskManager, thread);
+
+    taskManager.start();
+    assertFalse(taskManager.shouldShutdown());
+    // 2. When the task manager starts, there should have been a pending request on host1 and host2
+    assertEquals(2, allocator.getContainerRequestState().numPendingRequests());
+
+    // 3. Allocate an extra resource on host1 and no resource on host2 yet.
+    SamzaResource resource1 = new SamzaResource(1, 1000, "host1", "id1");
+    SamzaResource resource2 = new SamzaResource(1, 1000, "host1", "id2");
+    taskManager.onResourceAllocated(resource1);
+    taskManager.onResourceAllocated(resource2);
+
+    // 4. Wait for the container to start on host1 and immediately fail
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+    taskManager.onStreamProcessorLaunchSuccess(resource1);
+    assertEquals("host2", allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
+    assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
+
+    taskManager.onResourceCompleted(new SamzaResourceStatus(resource1.getResourceID(), "App Error", 1));
+    assertEquals(2, allocator.getContainerRequestState().numPendingRequests());
+
+    assertFalse(taskManager.shouldShutdown());
+    assertFalse(state.jobHealthy.get());
+    assertEquals(3, clusterResourceManager.resourceRequests.size());
+    assertEquals(0, clusterResourceManager.releasedResources.size());
+
+    // 5. Do not allocate any further resource on host1, and verify that the re-run of the container on host1 uses the
+    // previously allocated extra resource
+    SamzaResource resource3 = new SamzaResource(1, 1000, "host2", "id3");
+    taskManager.onResourceAllocated(resource3);
+
+    if (!allocator.awaitContainersStart(2, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+    taskManager.onStreamProcessorLaunchSuccess(resource3);
+
+    assertTrue(state.jobHealthy.get());
+  }
+
   @Test
   public void testDuplicateNotificationsDoNotAffectJobHealth() throws Exception {
     Config conf = getConfig();
@@ -397,16 +489,16 @@ public class TestContainerProcessManager {
     state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
 
     ContainerProcessManager taskManager = new ContainerProcessManager(
-            new MapConfig(conf),
-            state,
-            new MetricsRegistryMap(),
-            manager
+        new MapConfig(conf),
+        state,
+        new MetricsRegistryMap(),
+        clusterResourceManager
     );
 
     MockContainerAllocator allocator = new MockContainerAllocator(
-            manager,
-            conf,
-            state);
+        clusterResourceManager,
+        conf,
+        state);
     getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
 
     Thread thread = new Thread(allocator);
@@ -417,13 +509,14 @@ public class TestContainerProcessManager {
     assertFalse(taskManager.shouldShutdown());
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
 
-    SamzaResource container1 = new SamzaResource(1, 1000, "abc", "id1");
+    SamzaResource container1 = new SamzaResource(1, 1000, "host1", "id1");
     taskManager.onResourceAllocated(container1);
 
     // Allow container to run and update state
     if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
       fail("timed out waiting for the containers to start");
     }
+    taskManager.onStreamProcessorLaunchSuccess(container1);
     assertEquals(0, allocator.getContainerRequestState().numPendingRequests());
 
     // Create container failure - with ContainerExitStatus.DISKS_FAILED
@@ -433,25 +526,27 @@ public class TestContainerProcessManager {
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
     assertFalse(taskManager.shouldShutdown());
     assertFalse(state.jobHealthy.get());
-    assertEquals(2, manager.resourceRequests.size());
-    assertEquals(0, manager.releasedResources.size());
+    assertEquals(2, clusterResourceManager.resourceRequests.size());
+    assertEquals(0, clusterResourceManager.releasedResources.size());
     assertEquals(ResourceRequestState.ANY_HOST, allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
 
-    SamzaResource container2 = new SamzaResource(1, 1000, "abc", "id2");
+    SamzaResource container2 = new SamzaResource(1, 1000, "host1", "id2");
     taskManager.onResourceAllocated(container2);
 
     // Allow container to run and update state
     if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
       fail("timed out waiting for the containers to start");
     }
+    taskManager.onStreamProcessorLaunchSuccess(container2);
+
     assertTrue(state.jobHealthy.get());
 
     // Simulate a duplicate notification for container 1 with a different exit code
     taskManager.onResourceCompleted(new SamzaResourceStatus(container1.getResourceID(), "Disk failure", SamzaResourceStatus.PREEMPTED));
     // assert that a duplicate notification does not change metrics (including job health)
     assertEquals(state.redundantNotifications.get(), 1);
-    assertEquals(2, manager.resourceRequests.size());
-    assertEquals(0, manager.releasedResources.size());
+    assertEquals(2, clusterResourceManager.resourceRequests.size());
+    assertEquals(0, clusterResourceManager.releasedResources.size());
     assertTrue(state.jobHealthy.get());
   }
 
@@ -471,11 +566,11 @@ public class TestContainerProcessManager {
         new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
 
     MockContainerAllocator allocator = new MockContainerAllocator(
-        manager,
+        clusterResourceManager,
         conf,
         state);
     getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
@@ -483,12 +578,12 @@ public class TestContainerProcessManager {
     Thread thread = new Thread(allocator);
     getPrivateFieldFromTaskManager("allocatorThread", taskManager).set(taskManager, thread);
 
-    // Start the task manager
+    // Start the task clusterResourceManager
     taskManager.start();
     assertFalse(taskManager.shouldShutdown());
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
 
-    SamzaResource container1 = new SamzaResource(1, 1000, "abc", "id1");
+    SamzaResource container1 = new SamzaResource(1, 1000, "host1", "id1");
     taskManager.onResourceAllocated(container1);
 
     // Allow container to run and update state
@@ -496,50 +591,52 @@ public class TestContainerProcessManager {
       fail("timed out waiting for the containers to start");
     }
     assertEquals(0, allocator.getContainerRequestState().numPendingRequests());
-
+    taskManager.onStreamProcessorLaunchSuccess(container1);
     // Create container failure - with ContainerExitStatus.DISKS_FAILED
-    taskManager.onResourceCompleted(new SamzaResourceStatus(container1.getResourceID(), "Disk failure", SamzaResourceStatus.DISK_FAIL));
+    taskManager.onResourceCompleted(new SamzaResourceStatus(container1.getResourceID(), "App error", 1));
 
     // The above failure should trigger a container request
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
     assertFalse(taskManager.shouldShutdown());
     assertFalse(state.jobHealthy.get());
-    assertEquals(2, manager.resourceRequests.size());
-    assertEquals(0, manager.releasedResources.size());
+    assertEquals(2, clusterResourceManager.resourceRequests.size());
+    assertEquals(0, clusterResourceManager.releasedResources.size());
     assertEquals(ResourceRequestState.ANY_HOST, allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
 
-    SamzaResource container2 = new SamzaResource(1, 1000, "abc", "id2");
+    SamzaResource container2 = new SamzaResource(1, 1000, "host1", "id2");
     taskManager.onResourceAllocated(container2);
 
     // Allow container to run and update state
     if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
       fail("timed out waiting for the containers to start");
     }
+    taskManager.onStreamProcessorLaunchSuccess(container2);
 
     // Create container failure - with ContainerExitStatus.PREEMPTED
-    taskManager.onResourceCompleted(new SamzaResourceStatus(container2.getResourceID(), "Preemption",  SamzaResourceStatus.PREEMPTED));
-    assertEquals(3, manager.resourceRequests.size());
+    taskManager.onResourceCompleted(new SamzaResourceStatus(container2.getResourceID(), "Preemption", SamzaResourceStatus.PREEMPTED));
+    assertEquals(3, clusterResourceManager.resourceRequests.size());
 
     // The above failure should trigger a container request
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
     assertFalse(taskManager.shouldShutdown());
     assertFalse(state.jobHealthy.get());
     assertEquals(ResourceRequestState.ANY_HOST, allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
-    SamzaResource container3 = new SamzaResource(1, 1000, "abc", "id3");
+    SamzaResource container3 = new SamzaResource(1, 1000, "host1", "id3");
     taskManager.onResourceAllocated(container3);
 
     // Allow container to run and update state
     if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
       fail("timed out waiting for the containers to start");
     }
+    taskManager.onStreamProcessorLaunchSuccess(container3);
 
     // Create container failure - with ContainerExitStatus.ABORTED
     taskManager.onResourceCompleted(new SamzaResourceStatus(container3.getResourceID(), "Aborted", SamzaResourceStatus.ABORTED));
 
     // The above failure should trigger a container request
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
-    assertEquals(4, manager.resourceRequests.size());
-    assertEquals(0, manager.releasedResources.size());
+    assertEquals(4, clusterResourceManager.resourceRequests.size());
+    assertEquals(0, clusterResourceManager.releasedResources.size());
     assertFalse(taskManager.shouldShutdown());
     assertFalse(state.jobHealthy.get());
     assertEquals(ResourceRequestState.ANY_HOST, allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
@@ -556,7 +653,7 @@ public class TestContainerProcessManager {
         new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
     taskManager.start();
     SamzaResource container2 = new SamzaResource(1, 1024, "", "id0");
@@ -570,7 +667,7 @@ public class TestContainerProcessManager {
         new MapConfig(config),
         state,
         new MetricsRegistryMap(),
-        manager
+        clusterResourceManager
     );
     taskManager1.start();
     taskManager1.onResourceAllocated(container2);
