@@ -367,6 +367,11 @@ public class TestAbstractApplicationRunner {
       super(config);
     }
 
+    @Override
+    public void runTask() {
+      throw new UnsupportedOperationException("runTask is not supported in this test");
+    }
+
     @Override
     public void run(StreamApplication streamApp) {
       // do nothing. We're only testing the stream creation methods at this point.
