@@ -27,9 +27,11 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.function.BooleanSupplier;
+import com.google.common.collect.ImmutableList;
 import org.I0Itec.zkclient.IZkDataListener;
 import org.I0Itec.zkclient.ZkClient;
 import org.I0Itec.zkclient.ZkConnection;
+import org.I0Itec.zkclient.exception.ZkInterruptedException;
 import org.I0Itec.zkclient.exception.ZkNodeExistsException;
 import org.apache.commons.lang3.reflect.FieldUtils;
 import org.apache.samza.SamzaException;
@@ -46,7 +48,7 @@ import org.junit.BeforeClass;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
-
+import org.mockito.Mockito;
 
 public class TestZkUtils {
   private static EmbeddedZookeeper zkServer = null;
@@ -88,8 +90,9 @@ public class TestZkUtils {
 
   @After
   public void testTeardown() {
-    zkUtils.close();
-    zkClient.close();
+    if (zkClient != null) {
+      zkUtils.close();
+    }
   }
 
   private ZkUtils getZkUtils() {
@@ -118,6 +121,21 @@ public class TestZkUtils {
     Assert.assertEquals(1, zkUtils.getSortedActiveProcessorsZnodes().size());
   }
 
+  @Test
+  public void testGetActiveProcessorIdShouldReturnEmptyForNonExistingZookeeperNodes() {
+    List<String> processorsIDs = zkUtils.getActiveProcessorsIDs(ImmutableList.of("node1", "node2"));
+
+    Assert.assertEquals(0, processorsIDs.size());
+  }
+
+
+  @Test
+  public void testGetAllProcessorNodesShouldReturnEmptyForNonExistingZookeeperNodes() {
+    List<ZkUtils.ProcessorNode> processorsIDs = zkUtils.getAllProcessorNodes();
+
+    Assert.assertEquals(0, processorsIDs.size());
+  }
+
   @Test
   public void testZKProtocolVersion() {
     // first time connect, version should be set to ZkUtils.ZK_PROTOCOL_VERSION
@@ -392,6 +410,15 @@ public class TestZkUtils {
 
   }
 
+  @Test
+  public void testCloseShouldNotThrowZkInterruptedExceptionToCaller() {
+    ZkClient zkClient = Mockito.mock(ZkClient.class);
+    ZkUtils zkUtils = new ZkUtils(KEY_BUILDER, zkClient,
+            SESSION_TIMEOUT_MS, new NoOpMetricsRegistry());
+    Mockito.doThrow(new ZkInterruptedException(new InterruptedException())).when(zkClient).close();
+    zkUtils.close();
+  }
+
   public static boolean testWithDelayBackOff(BooleanSupplier cond, long startDelayMs, long maxDelayMs) {
     long delay = startDelayMs;
     while (delay < maxDelayMs) {
@@ -414,4 +441,26 @@ public class TestZkUtils {
       Assert.fail("Sleep was interrupted");
     }
   }
+  @Test
+  public void testgetNextJobModelVersion() {
+    // Set up the Zk base paths for testing.
+    ZkKeyBuilder keyBuilder = new ZkKeyBuilder("test");
+    String root = keyBuilder.getRootPath();
+    zkClient.deleteRecursive(root);
+    zkUtils.validatePaths(new String[]{root, keyBuilder.getJobModelPathPrefix(), keyBuilder.getJobModelVersionPath()});
+
+    String version = "1";
+    String oldVersion = "0";
+
+    // Set zkNode JobModelVersion to 1.
+    zkUtils.publishJobModelVersion(oldVersion, version);
+
+    Assert.assertEquals(version, zkUtils.getJobModelVersion());
+
+    // Publish JobModel with a higher version (2).
+    zkUtils.publishJobModel("2", new JobModel(new MapConfig(), new HashMap<>()));
+
+    // Get on the JobModel version should return 2, taking into account the published version 2.
+    Assert.assertEquals("3", zkUtils.getNextJobModelVersion(zkUtils.getJobModelVersion()));
+  }
 }
