@@ -29,7 +29,7 @@ import org.apache.samza.container.TaskName
 import org.apache.samza.storage.StoreProperties.StorePropertiesBuilder
 import org.apache.samza.system.SystemStreamMetadata.SystemStreamPartitionMetadata
 import org.apache.samza.system._
-import org.apache.samza.util.{SystemClock, Util}
+import org.apache.samza.util.{FileUtil, SystemClock}
 import org.junit.Assert._
 import org.junit.{After, Before, Test}
 import org.mockito.Matchers._
@@ -56,8 +56,8 @@ class TestTaskStorageManager extends MockitoSugar {
 
   @After
   def tearDownTestDirs() {
-    Util.rm(TaskStorageManagerBuilder.defaultStoreBaseDir)
-    Util.rm(TaskStorageManagerBuilder.defaultLoggedStoreBaseDir)
+    FileUtil.rm(TaskStorageManagerBuilder.defaultStoreBaseDir)
+    FileUtil.rm(TaskStorageManagerBuilder.defaultLoggedStoreBaseDir)
   }
 
   /**
@@ -125,7 +125,7 @@ class TestTaskStorageManager extends MockitoSugar {
     // Test 2: flush should update the offset file
     taskManager.flush()
     assertTrue(offsetFile.exists())
-    assertEquals("50", Util.readDataFromFile(offsetFile))
+    assertEquals("50", FileUtil.readWithChecksum(offsetFile))
 
     // Test 3: Update sspMetadata before shutdown and verify that offset file is updated correctly
     metadata = new SystemStreamMetadata("testStream", new java.util.HashMap[Partition, SystemStreamPartitionMetadata]() {
@@ -142,7 +142,7 @@ class TestTaskStorageManager extends MockitoSugar {
     taskManager.stop()
     assertTrue(storeFile.exists())
     assertTrue(offsetFile.exists())
-    assertEquals("100", Util.readDataFromFile(offsetFile))
+    assertEquals("100", FileUtil.readWithChecksum(offsetFile))
 
 
     // Test 4: Initialize again with an updated sspMetadata; Verify that it restores from the correct offset
@@ -274,7 +274,7 @@ class TestTaskStorageManager extends MockitoSugar {
   @Test
   def testLoggedStoreDirsWithOffsetFileAreNotDeletedInCleanBaseDirs() {
     val offsetFilePath = new File(TaskStorageManager.getStorePartitionDir(TaskStorageManagerBuilder.defaultLoggedStoreBaseDir, loggedStore, taskName), "OFFSET")
-    Util.writeDataToFile(offsetFilePath, "100")
+    FileUtil.writeWithChecksum(offsetFilePath, "100")
 
     val taskStorageManager = new TaskStorageManagerBuilder()
       .addStore(loggedStore, true)
@@ -296,7 +296,7 @@ class TestTaskStorageManager extends MockitoSugar {
     val storeDirectory = TaskStorageManager.getStorePartitionDir(TaskStorageManagerBuilder.defaultLoggedStoreBaseDir, loggedStore, taskName)
     val offsetFile = new File(storeDirectory, "OFFSET")
     offsetFile.createNewFile()
-    Util.writeDataToFile(offsetFile, "Test Offset Data")
+    FileUtil.writeWithChecksum(offsetFile, "Test Offset Data")
     offsetFile.setLastModified(0)
     val taskStorageManager = new TaskStorageManagerBuilder().addStore(store, false)
       .addStore(loggedStore, true)
@@ -315,7 +315,7 @@ class TestTaskStorageManager extends MockitoSugar {
   @Test
   def testOffsetFileIsRemovedInCleanBaseDirsForInMemoryLoggedStore() {
     val offsetFilePath = new File(TaskStorageManager.getStorePartitionDir(TaskStorageManagerBuilder.defaultLoggedStoreBaseDir, loggedStore, taskName), "OFFSET")
-    Util.writeDataToFile(offsetFilePath, "100")
+    FileUtil.writeWithChecksum(offsetFilePath, "100")
 
     val taskStorageManager = new TaskStorageManagerBuilder()
       .addStore(loggedStore, false)
@@ -352,7 +352,7 @@ class TestTaskStorageManager extends MockitoSugar {
 
     //Check conditions
     assertTrue("Offset file doesn't exist!", offsetFilePath.exists())
-    assertEquals("Found incorrect value in offset file!", "100", Util.readDataFromFile(offsetFilePath))
+    assertEquals("Found incorrect value in offset file!", "100", FileUtil.readWithChecksum(offsetFilePath))
   }
 
   /**
@@ -386,7 +386,7 @@ class TestTaskStorageManager extends MockitoSugar {
 
     //Check conditions
     assertTrue("Offset file doesn't exist!", offsetFilePath.exists())
-    assertEquals("Found incorrect value in offset file!", "100", Util.readDataFromFile(offsetFilePath))
+    assertEquals("Found incorrect value in offset file!", "100", FileUtil.readWithChecksum(offsetFilePath))
 
     assertTrue("Offset file got created for a store that is not persisted to the disk!!", !anotherOffsetPath.exists())
   }
@@ -416,7 +416,7 @@ class TestTaskStorageManager extends MockitoSugar {
 
     //Check conditions
     assertTrue("Offset file doesn't exist!", offsetFilePath.exists())
-    assertEquals("Found incorrect value in offset file!", "100", Util.readDataFromFile(offsetFilePath))
+    assertEquals("Found incorrect value in offset file!", "100", FileUtil.readWithChecksum(offsetFilePath))
 
     //Invoke test method again
     taskStorageManager.flush()
@@ -430,7 +430,7 @@ class TestTaskStorageManager extends MockitoSugar {
     val partition = new Partition(0)
 
     val offsetFilePath = new File(TaskStorageManager.getStorePartitionDir(TaskStorageManagerBuilder.defaultLoggedStoreBaseDir, loggedStore, taskName) + File.separator + "OFFSET")
-    Util.writeDataToFile(offsetFilePath, "100")
+    FileUtil.writeWithChecksum(offsetFilePath, "100")
 
     val mockSystemAdmin = mock[SystemAdmin]
     var mockSspMetadata = Map("testStream" -> new SystemStreamMetadata("testStream" , Map(partition -> new SystemStreamPartitionMetadata("20", "139", "140")).asJava))
@@ -449,7 +449,7 @@ class TestTaskStorageManager extends MockitoSugar {
 
     //Check conditions
     assertTrue("Offset file doesn't exist!", offsetFilePath.exists())
-    assertEquals("Found incorrect value in offset file!", "139", Util.readDataFromFile(offsetFilePath))
+    assertEquals("Found incorrect value in offset file!", "139", FileUtil.readWithChecksum(offsetFilePath))
 
     // Flush again
     mockSspMetadata = Map("testStream" -> new SystemStreamMetadata("testStream" , Map(partition -> new SystemStreamPartitionMetadata("20", "193", "194")).asJava))
@@ -461,7 +461,7 @@ class TestTaskStorageManager extends MockitoSugar {
 
     //Check conditions
     assertTrue("Offset file doesn't exist!", offsetFilePath.exists())
-    assertEquals("Found incorrect value in offset file!", "193", Util.readDataFromFile(offsetFilePath))
+    assertEquals("Found incorrect value in offset file!", "193", FileUtil.readWithChecksum(offsetFilePath))
   }
 
   @Test
@@ -556,7 +556,7 @@ class TestTaskStorageManager extends MockitoSugar {
     if (writeOffsetFile) {
       val offsetFile = new File(storeDirectory, "OFFSET")
       if (fileOffset != null) {
-        Util.writeDataToFile(offsetFile, fileOffset)
+        FileUtil.writeWithChecksum(offsetFile, fileOffset)
       } else {
         // Write garbage to produce a null result when it's read
         val fos = new FileOutputStream(offsetFile)
@@ -715,10 +715,10 @@ class TaskStorageManagerBuilder extends MockitoSugar {
       changeLogSystemStreams = changeLogSystemStreams,
       changeLogStreamPartitions = changeLogStreamPartitions,
       streamMetadataCache = streamMetadataCache,
-      storeBaseDir = storeBaseDir,
+      nonLoggedStoreBaseDir = storeBaseDir,
       loggedStoreBaseDir = loggedStoreBaseDir,
       partition = partition,
-      systemAdmins = systemAdmins,
+      systemAdmins = new SystemAdmins(systemAdmins.asJava),
       new StorageConfig(new MapConfig()).getChangeLogDeleteRetentionsInMs,
       SystemClock.instance
     )
