@@ -250,12 +250,15 @@ class KafkaConfig(config: Config) extends ScalaMapConfig(config) {
     val kafkaChangeLogProperties = new Properties
 
     val appConfig = new ApplicationConfig(config)
-    if (appConfig.getAppMode == ApplicationMode.STREAM) {
-      kafkaChangeLogProperties.setProperty("cleanup.policy", "compact")
-    } else{
-      kafkaChangeLogProperties.setProperty("cleanup.policy", "compact,delete")
-      kafkaChangeLogProperties.setProperty("retention.ms", String.valueOf(KafkaConfig.DEFAULT_RETENTION_MS_FOR_BATCH))
-    }
+    // SAMZA-1600: do not use the combination of "compact,delete" as cleanup policy until we pick up Kafka broker 0.11.0.3,
+    // 1.0.2, or 1.1.0 (see KAFKA-6568)
+    // if (appConfig.getAppMode == ApplicationMode.STREAM) {
+    //  kafkaChangeLogProperties.setProperty("cleanup.policy", "compact")
+    // } else{
+    //  kafkaChangeLogProperties.setProperty("cleanup.policy", "compact,delete")
+    //  kafkaChangeLogProperties.setProperty("retention.ms", String.valueOf(KafkaConfig.DEFAULT_RETENTION_MS_FOR_BATCH))
+    // }
+    kafkaChangeLogProperties.setProperty("cleanup.policy", "compact")
     kafkaChangeLogProperties.setProperty("segment.bytes", KafkaConfig.CHANGELOG_DEFAULT_SEGMENT_SIZE)
     kafkaChangeLogProperties.setProperty("delete.retention.ms", String.valueOf(new StorageConfig(config).getChangeLogDeleteRetentionInMs(name)))
     filteredConfigs.asScala.foreach { kv => kafkaChangeLogProperties.setProperty(kv._1, kv._2) }
