@@ -22,7 +22,7 @@ package org.apache.samza.storage.kv
 import org.apache.samza.util.Logging
 import org.apache.samza.storage.{StoreProperties, StorageEngine}
 import org.apache.samza.system.IncomingMessageEnvelope
-import org.apache.samza.util.TimerUtils
+import org.apache.samza.util.TimerUtil
 
 import scala.collection.JavaConverters._
 
@@ -37,7 +37,7 @@ class KeyValueStorageEngine[K, V](
   rawStore: KeyValueStore[Array[Byte], Array[Byte]],
   metrics: KeyValueStorageEngineMetrics = new KeyValueStorageEngineMetrics,
   batchSize: Int = 500,
-  val clock: () => Long = { System.nanoTime }) extends StorageEngine with KeyValueStore[K, V] with TimerUtils with Logging {
+  val clock: () => Long = { System.nanoTime }) extends StorageEngine with KeyValueStore[K, V] with TimerUtil with Logging {
 
   var count = 0
 
@@ -99,6 +99,8 @@ class KeyValueStorageEngine[K, V](
    * batching updates to underlying raw store to notAValidEvent wrapping functions for efficiency.
    */
   def restore(envelopes: java.util.Iterator[IncomingMessageEnvelope]) {
+    info("Restoring entries for store " + metrics.storeName)
+
     val batch = new java.util.ArrayList[Entry[Array[Byte], Array[Byte]]](batchSize)
 
     for (envelope <- envelopes.asScala) {
@@ -114,10 +116,14 @@ class KeyValueStorageEngine[K, V](
 
       if (valBytes != null) {
         metrics.restoredBytes.inc(valBytes.size)
+        metrics.restoredBytesGauge.set(metrics.restoredBytesGauge.getValue + valBytes.size)
       }
 
       metrics.restoredBytes.inc(keyBytes.size)
-      metrics.restoredMessages.inc
+      metrics.restoredBytesGauge.set(metrics.restoredBytesGauge.getValue + keyBytes.size)
+
+      metrics.restoredMessages.inc()
+      metrics.restoredMessagesGauge.set(metrics.restoredMessagesGauge.getValue + 1)
       count += 1
 
       if (count % 1000000 == 0) {
@@ -125,6 +131,8 @@ class KeyValueStorageEngine[K, V](
       }
     }
 
+    info(count + " total entries restored.")
+
     if (batch.size > 0) {
       rawStore.putAll(batch)
     }
@@ -152,4 +160,11 @@ class KeyValueStorageEngine[K, V](
   }
 
   override def getStoreProperties: StoreProperties = storeProperties
+
+  override def snapshot(from: K, to: K): KeyValueSnapshot[K, V] = {
+    updateTimer(metrics.snapshotNs) {
+      metrics.snapshots.inc
+      wrapperStore.snapshot(from, to)
+    }
+  }
 }
